@@ -1,20 +1,27 @@
 import string
 import random
+import logging
+import lasso
 
 from django.db import transaction
 from django.contrib.auth.models import User, UserManager
-from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext as _
 
-import lasso
+from authentic2.saml.common import \
+    lookup_federation_by_name_id_and_provider_id, add_federation, \
+    get_idp_options_policy
+from authentic2.saml.models import LIBERTY_SESSION_DUMP_KIND_SP, \
+    LibertySessionDump, LibertyProvider
+from authentic2.authsaml2.models import SAML2TransientUser
+
+logger = logging.getLogger('authentic2.authsaml2.backends')
 
-from authentic2.saml.common import *
-from models import SAML2TransientUser
 
 class AuthenticationError(Exception):
     pass
 
+
 class AuthSAML2Backend:
     def logout_list(self, request):
         pid = None
@@ -22,29 +29,53 @@ class AuthSAML2Backend:
             objects.filter(django_session_key=request.session.session_key,
                     kind=LIBERTY_SESSION_DUMP_KIND_SP)
         if not q:
+            logger.debug('logout_list: no LibertySessionDump found')
             return []
+        '''
+            We deal with a single IdP session
+        '''
         try:
-            pid = lasso.Session().newFromDump(q[0].session_dump). \
-                get_assertions().keys()[0]
-            provider_ids = set([pid])
+            provider_id = lasso.Session(). \
+                newFromDump(q[0].session_dump.encode('utf-8')). \
+                    get_assertions().keys()[0]
         except:
-            pass
-        if not pid:
             return []
-        name = pid
+        if not provider_id:
+            return []
+        logger.debug('logout_list: Found session for %s' % provider_id)
+        name = provider_id
+        provider = None
         try:
-            name = models.LibertyProvider.objects.get(entity_id=pid).name
-        except models.LibertyProvider.DoesNotExist:
-            pass
-        import saml2_endpoints
-        code = '<div>'
-        code += _('Sending logout to %(pid)s....') % { 'pid': pid }
-        code += '<iframe src="%s" marginwidth="0" marginheight="0" \
-scrolling="no" style="border: none" width="16" height="16"></iframe></div>' % \
-                reverse(saml2_endpoints.sp_slo, args=[pid])
-        return [ code ]
+            provider = LibertyProvider.objects.get(entity_id=provider_id)
+            name = provider.name
+        except LibertyProvider.DoesNotExist:
+            logger.error('logout_list: session found for unknown provider %s' \
+                % provider_id)
+            return []
+
+        policy =  get_idp_options_policy(provider)
+        if not policy:
+            logger.error('logout_list: No policy found for %s' % provider_id)
+            return []
+        elif not policy.forward_slo:
+            logger.info('logout_list: %s configured to not reveive slo' \
+                % provider_id)
+            return []
+        else:
+            import saml2_endpoints
+            code = '<div>'
+            code += _('Sending logout to %(pid)s....') % { 'pid': name or provider_id }
+            code += '<iframe src="%s?provider_id=%s" marginwidth="0" marginheight="0" \
+    scrolling="no" style="border: none" width="16" height="16"></iframe></div>' \
+                % (reverse(saml2_endpoints.sp_slo,
+                    args=[provider_id]), provider_id)
+            return [ code ]
+
 
 class AuthSAML2PersistentBackend:
+    supports_object_permissions = False
+    supports_anonymous_user = False
+
     def authenticate(self, name_id=None, provider_id=None):
         '''Authenticate persistent NameID'''
         if not name_id or not provider_id:# or not name_id.nameQualifier:
@@ -53,8 +84,7 @@ class AuthSAML2PersistentBackend:
         fed = lookup_federation_by_name_id_and_provider_id(name_id, provider_id)
         if fed is None:
             return None
-        fed.user.backend = \
-            'authentic2.authsaml2.backends.AuthSAML2PersistentBackend'
+        fed.user.backend = '%s.%s' % (__name__, self.__class__.__name__)
         return fed.user
 
     def get_user(self, user_id):
@@ -84,6 +114,9 @@ class AuthSAML2PersistentBackend:
         return user
 
 class AuthSAML2TransientBackend:
+    supports_object_permissions = False
+    supports_anonymous_user = False
+
     def authenticate(self, name_id=None):
         '''Create temporary user for transient NameID'''
         if not name_id or \
