@@ -5,6 +5,7 @@ import httplib
 import logging
 import re
 import datetime
+import time
 
 import lasso
 from django.template import RequestContext
@@ -13,14 +14,22 @@ from django.http import HttpResponseRedirect, Http404, HttpResponse
 from django.shortcuts import render_to_response
 from django.utils.translation import ugettext as _
 from django.db import transaction
+from django.core.exceptions import ValidationError
+from django.core.exceptions import ObjectDoesNotExist
 
-from models import *
-import models
-import saml2utils
-import saml11utils
-import authentic2.utils
+from authentic2.saml.models import LibertyFederation, LibertyIdentityDump, LibertyProvider, \
+    LibertyManageDump, LibertySessionDump, LibertyServiceProvider, \
+    LibertyIdentityProvider, LibertySessionSP, IdPOptionsSPPolicy, \
+    SPOptionsIdPPolicy, \
+    AuthorizationSPPolicy, AuthorizationAttributeMapping, \
+    LIBERTY_SESSION_DUMP_KIND_IDP
+from authentic2.saml import models
+from authentic2.saml import saml2utils
+from authentic2.saml import saml11utils
 
 from authentic2.authsaml2 import signals
+from authentic2.http_utils import get_url
+from .. import nonce
 
 AUTHENTIC_STATUS_CODE_NS = "http://authentic.entrouvert.org/status_code/"
 AUTHENTIC_STATUS_CODE_UNKNOWN_PROVIDER = AUTHENTIC_STATUS_CODE_NS + \
@@ -35,13 +44,21 @@ AUTHENTIC_STATUS_CODE_MISSING_DESTINATION = AUTHENTIC_STATUS_CODE_NS + \
     "MissingDestination"
 AUTHENTIC_STATUS_CODE_INTERNAL_SERVER_ERROR = AUTHENTIC_STATUS_CODE_NS + \
     "InternalServerError"
+AUTHENTIC_STATUS_CODE_UNAUTHORIZED = AUTHENTIC_STATUS_CODE_NS + \
+    "Unauthorized"
 
-logger = logging.getLogger('authentic2.saml')
+logger = logging.getLogger('authentic2.saml.common')
+
+# timeout for messages and assertions issue instant
+NONCE_TIMEOUT = getattr(settings, 'SAML2_NONCE_TIMEOUT',
+        getattr(settings, 'NONCE_TIMEOUT', 30))
+# do we check id on SAML2 messages ?
+CHECKS_ID = getattr(settings, 'SAML2_CHECKS_ID', True)
 
 def get_soap_message(request, on_error_raise = True):
     '''Verify that POST content looks like a SOAP message and returns it'''
     if request.method != 'POST' or \
-            request.META['CONTENT_TYPE'] != 'text/xml':
+            'text/xml' not in request.META['CONTENT_TYPE']:
        if on_error_raise:
            raise Http404(_('Only SOAP messages here'))
        else:
@@ -90,7 +107,7 @@ def create_saml2_server(request, metadata, idp_map=None, sp_map=None, options={}
     '''Create a lasso Server object for using with a profile'''
     server = lasso.Server.newFromBuffers(get_saml2_metadata(request, metadata,
         idp_map=idp_map, sp_map=sp_map, options=options),
-        settings.SAML_PRIVATE_KEY)
+        settings.SAML_SIGNATURE_PRIVATE_KEY)
     if not server:
         raise Exception('Cannot create LassoServer object')
     return server
@@ -157,6 +174,38 @@ def return_saml2(request, profile, field_name, title = ''):
     else:
         raise TypeError('profile do not contain a response')
 
+def check_id_and_issue_instant(request_response_or_assertion, now=None):
+    '''
+       Check that issue instant is not older than a timeout and also checks
+       that the id has never been seen before.
+
+       Nonce are cached for two times the relative timeout length of the issue
+       instant.
+    '''
+    if now is None:
+        now = datetime.datetime.utcnow()
+    try:
+        issue_instant = request_response_or_assertion.issueInstant
+        issue_instant = iso8601_to_datetime(issue_instant)
+        delta = datetime.timedelta(seconds=NONCE_TIMEOUT)
+        if not (now - delta <= issue_instant < now + delta):
+            logger.warning('IssueInstant %s not in the interval [%s, %s[',
+                    issue_instant, now-delta, now+delta)
+            return False
+    except ValueError:
+        logger.error('Unable to parse an IssueInstant: %r', issue_instant)
+        return False
+    if CHECKS_ID:
+        _id = request_response_or_assertion.id
+        if _id is None:
+            logger.warning('missing ID')
+            return False
+        if not nonce.accept_nonce(_id, 'SAML', 2*NONCE_TIMEOUT):
+            logger.warning("ID '%r' already used, request/response/assertion "
+                    "refused", _id)
+            return False
+    return True
+
 # ID-FF 1.2 methods
 def get_idff12_metadata(request, metadata):
     '''Produce SAMLv1.1 metadata for our ID-FF 1.2 IdP
@@ -174,13 +223,13 @@ def get_idff12_metadata(request, metadata):
             'SoapEndpoint': '/soap',
             'SingleSignOn': (('/sso',), sso_protocol_profiles)
           }
-    options = { 'key': settings.SAML_SIGNING_KEY }
+    options = { 'key': settings.SAML_SIGNATURE_PUBLIC_KEY }
     metagen.add_idp_descriptor(map, options)
     return str(metagen)
 
 def create_idff12_server(request, metadata):
     server = lasso.Server.newFromBuffers(get_idff12_metadata(request,
-        metadata), settings.SAML_PRIVATE_KEY)
+        metadata), settings.SAML_SIGNATURE_PRIVATE_KEY)
     if not server:
         raise Exception('Cannot create LassoServer object')
     return server
@@ -272,15 +321,11 @@ def save_session(request, login, session_key=None,
 
 def delete_session(request):
     '''Delete all liberty sessions for a django session'''
-    all_sessions = LibertySessionDump.objects.get(django_session_key = request.session.session_key)
     try:
-        all_sessions.delete()
-    except:
-        try:
-            for s in all_sessions:
-                s.delete()
-        except:
-            pass
+        LibertySessionDump.objects.\
+            filter(django_session_key = request.session.session_key).delete()
+    except Exception, e:
+        logger.error('delete_session: Exception %s' % str(e))
 
 def save_manage(request, manage):
     if not request or not manage:
@@ -296,15 +341,18 @@ def get_manage_dump(request):
 
 @transaction.commit_on_success
 def retrieve_metadata_and_create(request, provider_id, sp_or_idp):
+    logger.debug('trying to load %s from wkl' % provider_id)
     if not provider_id.startswith('http'):
+        logger.debug('not an http url, failing')
         return None
     # Try the WKL
     try:
-        metadata = urllib.urlopen(provider_id).read()
-    except:
-        logging.error('SAML metadata autoload: failure to retrieve metadata \
-for entity id %r' % provider_id)
+        metadata = get_url(provider_id)
+    except Exception, e:
+        logging.error('SAML metadata autoload: failure to retrieve metadata '
+                'for entity id %r: %s' % (provider_id, e))
         return None
+    logger.debug('loaded %d bytes' % len(metadata))
     try:
         metadata = unicode(metadata, 'utf8')
     except:
@@ -322,8 +370,9 @@ for entity id %r are invalid, %s' % (provider_id, e.args))
         logging.exception('SAML metadata autoload: retrieved metadata validation raised an unknown exception')
         return None
     p.save()
+    logger.debug('%s saved' % p)
     if sp_or_idp == 'sp':
-        s = LibertyServiceProvider(liberty_provider=p, enabled=True, ask_user_consent=True)
+        s = LibertyServiceProvider(liberty_provider=p, enabled=True)
         s.save()
     elif sp_or_idp == 'idp':
         s = LibertyIdentityProvider(liberty_provider=p, enabled=True)
@@ -430,7 +479,7 @@ def lookup_federation_by_user(user, qualifier):
     fed = LibertyFederation.objects.filter(user=user, name_id_qualifier=qualifier)
     if fed and fed.count()>1:
         # TODO: delete all but the last record
-        raise Exception('Unconsistent federation record for %s' % ni)
+        raise Exception('Unconsistent federation record for %s' % qualifier)
     if not fed:
         return None
     return fed[0]
@@ -449,32 +498,23 @@ def get_idp_user_federated_list(request):
     user = request.user
     if request.user.is_anonymous():
         return None
-    providers_list = get_idp_list()
-    p_list = []
-    for p in providers_list:
-        fed = lookup_federation_by_user(user, p.entity_id)
-        if fed:
-            p_list.append(p)
-    return p_list
+    return [p for p in get_idp_list() \
+        if lookup_federation_by_user(user, p.entity_id)]
 
 def get_idp_user_not_federated_list(request):
     user = request.user
     if request.user.is_anonymous():
         return None
-    providers_list = get_idp_list()
-    p_list = []
-    for p in providers_list:
-        fed = lookup_federation_by_user(user, p.entity_id)
-        if not fed:
-            p_list.append(p)
-    return p_list
+    return [p for p in get_idp_list() \
+        if not lookup_federation_by_user(user, p.entity_id)]
+
 
 # The session_index is the "session on the IdP" identifiers
-# One identifier is dedicated for each sp for each user session 
+# One identifier is dedicated for each sp for each user session
 # to not be a factor of linkability between sp
 # (like the nameId dedicated for each sp)
 # A same user IdP session is thus made of as many session index as SP having received an auth a8n
-# The session index is only useful to maintain 
+# The session index is only useful to maintain
 # the coherence between the sessions on the IdP and on the SP
 # for the global logout:
 # If one session is broken somewhere, when a session is restablished there
@@ -535,14 +575,11 @@ def get_session_index(request):
         return None
 
 def remove_liberty_session_sp(request):
-    if not request:
-        return None
     try:
-        ss = LibertySessionSP.objects.filter(django_session_key=request.session.session_key)
-        for s in ss:
-            s.delete()
-    except:
-        return None
+        LibertySessionSP.objects.\
+            filter(django_session_key=request.session.session_key).delete()
+    except Exception, e:
+        logger.error('remove_liberty_session_sp: Exception %s' % str(e))
 
 def get_provider_of_active_session(request):
     if not request:
@@ -576,17 +613,22 @@ def soap_call(url, msg, client_cert = None):
         host, query = urllib.splithost(url[6:])
         conn = httplib.HTTPSConnection(host,
                 key_file = client_cert, cert_file = client_cert)
+    logger.debug('soap_call: host %s' % host)
+    logger.debug('soap_call: query %s' % query)
+    logger.debug('soap_call: msg %s' % msg)
     try:
         conn.request('POST', query, msg, {'Content-Type': 'text/xml'})
         response = conn.getresponse()
     except Exception, err:
         logging.error('SOAP error (on %s): %s' % (url, err))
         raise SOAPException(url)
+    logger.debug('soap_call: response %s' % str(response))
     try:
         data = response.read()
     except Exception, err:
         logging.error('SOAP error (on %s): %s' % (url, err))
         raise SOAPException(url)
+    logger.debug('soap_call: data %s' % str(data))
     conn.close()
     if response.status not in (200, 204): # 204 ok for federation termination
         logging.warning('SOAP error (%s) (on %s)' % (response.status, url))
@@ -631,6 +673,12 @@ def error_page(request, message, back = None, logger = None):
     return render_to_response('error.html', {'msg': message, 'back': back},
             context_instance=RequestContext(request))
 
+def redirect_next(request, next):
+    if next:
+        return HttpResponseRedirect(next)
+    else:
+        return None
+
 def soap_fault(request, faultcode='soap:Client', faultstring=None):
     if faultstring:
         faultstring = '\n        <faultstring>%s</faultstring>\n' % faultstring
@@ -665,14 +713,34 @@ def get_idp_options_policy(provider):
         pass
     return None
 
+def get_sp_options_policy(provider):
+    try:
+        return SPOptionsIdPPolicy.objects.get(name='All', enabled=True)
+    except SPOptionsIdPPolicy.DoesNotExist:
+        pass
+    try:
+        if provider.service_provider.enable_following_sp_options_policy:
+            if provider.service_provider.sp_options_policy:
+                return provider.service_provider.sp_options_policy
+    except:
+        pass
+    try:
+        return SPOptionsIdPPolicy.objects.get(name='Default', enabled=True)
+    except SPOptionsIdPPolicy.DoesNotExist:
+        pass
+    return None
+
 def get_authorization_policy(provider):
     try:
         return AuthorizationSPPolicy.objects.get(name='All', enabled=True)
     except AuthorizationSPPolicy.DoesNotExist:
         pass
-    if provider.identity_provider.enable_following_authorization_policy:
-        if provider.identity_provider.authorization_policy:
-            return provider.identity_provider.authorization_policy
+    try:
+        if provider.identity_provider.enable_following_authorization_policy:
+            if provider.identity_provider.authorization_policy:
+                return provider.identity_provider.authorization_policy
+    except:
+        pass
     try:
         return AuthorizationSPPolicy.objects.get(name='Default', enabled=True)
     except AuthorizationSPPolicy.DoesNotExist:
@@ -683,7 +751,7 @@ def attributesMatch(attributes, attributes_to_check):
     attrs = AuthorizationAttributeMapping. \
         objects.filter(map=attributes_to_check)
     for attr in attrs:
-        if not attributes.has_key(attr.attribute_name):
+        if not attr.attribute_name in attributes:
             raise Exception('Attribute %s not provided' %attr.attribute_name)
         if not attr.attribute_value in attributes[attr.attribute_name]:
             return False
@@ -703,3 +771,20 @@ def authz_decision_cb(sender, request=None, attributes={},
 
 signals.authz_decision.connect(authz_decision_cb,
     dispatch_uid='authz_decision_on_attributes')
+
+def get_session_not_on_or_after(assertion):
+    '''Extract the minimal value for the SessionNotOnOrAfter found in the given
+       assertion AuthenticationStatement(s).
+    '''
+    session_not_on_or_afters = []
+    if hasattr(assertion, 'authnStatement'):
+        for authn_statement in assertion.authnStatement:
+            if authn_statement.sessionNotOnOrAfter:
+                value = authn_statement.sessionNotOnOrAfter
+                try:
+                    session_not_on_or_afters.append(iso8601_to_datetime(value))
+                except ValueError:
+                    logging.getLogger(__name__).error('unable to parse SessionNotOnOrAfter value %s, will use default value for session length.', value)
+    if session_not_on_or_afters:
+        return reduce(min, session_not_on_or_afters)
+    return None
